#################################################################################
# Created       : Martin Foster
# Modified      : 18-Mar-2006
#################################################################################
#
# Web - Module part of Ethereal Realms, designed to deal with the complexities of
#            the LWP packages.
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 4-3526 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

package Ethereal::Web;						# Package name

use strict;							# Strict variable enforcement



#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle


	#####################
	# Program Area

	# Generate self
	my $self     = {
		AGENT => undef
	 };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


####################
# Fetch
#
# Simply fetches a given address and returns the contents.   This also hides the
# complexity associated with using LWP::UserAgent directly.

sub Fetch
{
	####################
	# Data members
	my $self   = shift;					# Reference to self
	my $addr   = shift;					# Address or hostname
	my $type   = shift;					# Type of request to make

	my $ua     = $self->{'AGENT'};				# User agent
	my $reply;						# Response from server


	####################
	# Program area

	# Create instance and set
	unless (defined($ua))
	{
		# Generate new agent
		$ua = $self->UserAgent();

		# Copy for reuse
		$self->{'AGENT'} = $ua;
	}

	#
	# Fetch the document
	$reply = $ua->post($addr, \@_) if ($type eq 'post');
	$reply = $ua->get($addr, \@_)  if ($type eq 'get');
	$reply = $ua->head($addr, \@_) if ($type eq 'head');

	# Success 
	if ($reply->is_success())
	{
		# Header return
		return $reply->header('Content-Type') if ($type eq 'head');

		# Returns the content and status line
		return $reply->content(), $reply->status_line(); 
	}

	# Failed
	else
	{
		# Return no content and status line
		return undef, $reply->status_line();
	}
}

# Fetch Head
sub Fetch_Head
{ return Fetch(shift, shift, 'head', @_); }

# Fetch Get
sub Fetch_Get
{ return Fetch(shift, shift, 'get', @_); }

# Fetch Posts
sub Fetch_Post
{ return Fetch(shift, shift, 'post', @_); }




####################
# Request
#
# Simply fetches a given address and returns the contents and
# places them into a file.

sub Request
{
	####################
	# Data members
	my $self   = shift;					# Reference to self
	my $addr   = shift;					# Address or hostname
	my $file   = shift;					# File name

	my $req;						# Request module
	my $reply;						# Reply received

	my $ua     = $self->{'AGENT'};				# User agent


	####################
	# Program area

	# Create instance and set
	unless (defined($ua))
	{
		# Generate new agent
		$ua = $self->UserAgent();

		# Copy for reuse
		$self->{'AGENT'} = $ua;
	}

	# Create the request
        require HTTP::Request;

	# Create instance
	$req = HTTP::Request->new(GET => $addr);


	#
	# Fetch the document
	$reply = $ua->simple_request($req, $file);

	# Return information
	return $reply->status_line();
}


####################
# User Agent
#
# Make use of a choice of user agents if available and then uses the most
# secure one first.

sub UserAgent
{
	####################
	# Data members
	my $self   = shift;					# Reference to self

	my $handle;						# User agent
	my $agent;						# Singular agent
	my @agents = qw(LWPx::ParanoidAgent LWP::UserAgent);	# List of agents

	####################
	# Program area

	# Loop through
	foreach $agent (@agents)
	{
		# Evaluate and check
		eval("use $agent\; \$handle = new $agent\;");

		# No need to loop if it passes
		last unless ($@);
	}

	# Establish basic variables
	$handle->timeout(10);
	$handle->max_redirect(2);
	$handle->env_proxy();

	return $handle;
}

1;								# Return true
